/*
 * Copyright (C) 2012 CERN (www.cern.ch)
 * Author: Aurelio Colosimo
 * Based on ptp-noposix project (see AUTHORS for details)
 *
 * Released to the public domain
 */

#include <ppsi/ppsi.h>

/*
 * We enter here from WRS_CALIBRATION.  If master we wait for
 * a CALIBRATE message, if slave we wait for LINK_ON.
 */
int wr_calibrated(struct pp_instance *ppi, void *buf, int len)
{
	struct wr_dsport *wrp = WR_DSPOR(ppi);
	MsgSignaling wrsig_msg;
	int enable = 0;

	if (ppi->is_new_state) {
		wrp->wrStateRetry = WR_STATE_RETRY;
		__pp_timeout_set(ppi, PP_TO_EXT_0, WR_CALIBRATED_TIMEOUT_MS*(WR_STATE_RETRY+1));
		enable = 1;
	} else {
		int rms=pp_next_delay_1(ppi, PP_TO_EXT_0);
		if ( rms==0 || rms<(wrp->wrStateRetry*WR_CALIBRATED_TIMEOUT_MS)) {
		/*
		 * FIXME: We should implement a retry by re-sending
		 * the "calibrated" message, moving it here from the
		 * previous state (sub-state 8 of "state-wr-calibration"
		 */
			if (wr_handshake_retry(ppi))
				enable = 1;
			else
				return 0; /* non-wr already */
		}
	}

	if (enable){
		msg_issue_wrsig(ppi, CALIBRATED);
    }

	if (ppi->received_ptp_header.messageType == PPM_SIGNALING) {
		msg_unpack_wrsig(ppi, buf, &wrsig_msg,
			 &(wrp->msgTmpWrMessageID));

		if ((wrp->msgTmpWrMessageID == CALIBRATE) &&
			(wrp->wrMode == WR_MASTER))
			ppi->next_state = WRS_RESP_CALIB_REQ;
		else if ((wrp->msgTmpWrMessageID == WR_MODE_ON) &&
			(wrp->wrMode == WR_SLAVE))
			ppi->next_state = WRS_WR_LINK_ON;
	}

	ppi->next_delay = pp_next_delay_1(ppi,PP_TO_EXT_0)-wrp->wrStateRetry*WR_CALIBRATED_TIMEOUT_MS;
	return 0;
}
